/* WORLD -- by Mark Cederholm

 * Reads the .DOC file of an IDRISI image
   and writes a .TFW file accordingly

 * Version 1.0, 4/9/98

 * Usage:  WORLD [infile] [outfile]

 * Arguments:   infile = input documentation file (.DOC extension assumed)
               outfile = output world file (.TFW extension assumed)

/*********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{

   /**** initialize environment **************************************/

   FILE *infile, *outfile;
   char doc[81];         /* .DOC file name                           */
   char tfw[81];         /* .TFW file name                           */
   char input[81];       /* input line                               */
   char temp[81];        /* working string                           */
   char title[81];       /* quad title                               */
   char *tok;            /* current token                            */
   double x0, y0;        /* min x and Y                              */
   double x1, y1;        /* max x and Y                              */
   int row, col;         /* num of rows and columns                  */
   int tmp;              /* temp variable                            */
   double a, e, c, f;    /* output values                            */

   /* check # of arguments */

   if (argc != 3)
   {
      printf("Usage:  WORLD [doc_file] [tfw_file]\n");
      return 0;
   };

   /* initialize variables */

   strcpy(doc, argv[1]);
   strcpy(tfw, argv[2]);
   strcat(doc, ".doc");
   strcat(tfw, ".tfw");

   /* open files */

   if ((infile = fopen(doc,"r")) == NULL)
   {
      printf("Error opening input file %s.\n", doc);
      return 0;
   };
   if ((outfile = fopen(tfw,"w")) == NULL)
   {
      printf("Error opening output file %s.\n", tfw);
      return 0;
   };

   /* read DOC file */

   while (fgets(input, 80, infile) != NULL)
   {

      strcpy(temp,"columns");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         col = atoi(tok);
         continue;
      }

      strcpy(temp,"rows");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         row = atoi(tok);
         continue;
      }

      strcpy(temp,"min. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         x0 = atof(tok);
         continue;
      }

      strcpy(temp,"min. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         y0 = atof(tok);
         continue;
      }

      strcpy(temp,"max. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         x1 = atof(tok);
         continue;
      }

      strcpy(temp,"max. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         y1 = atof(tok);
         continue;
      }

   }

   /* calculate world file parameters */

   a = (x1-x0)/col;
   e = (y0-y1)/row;
   c = x0+(a/2);
   f = y1+(e/2);

   fprintf(outfile,"%f\n", a);
   fprintf(outfile,"0\n");
   fprintf(outfile,"0\n");
   fprintf(outfile,"%f\n", e);
   fprintf(outfile,"%f\n", c);
   fprintf(outfile,"%f\n", f);

   return 0;

}

